let data;
let svg;

let canvasWidth = 800;
let canvasHeight = 500;
let marginX = 200;
let marginY = 140;
let w = canvasWidth - marginX;
let h = canvasHeight - marginY;

const xLabel = "Year";
const yLabel = "Share of production (%)";
const colorLabel = "Change on last year (%)";

const fetchCSV = async() => {
    //fetch csv data
    const pathToCSV = "Europe_Power_Sector_Renewables_2012_to_2020.csv";
    
    data = await d3.csv(pathToCSV, d => {
        return {
            year: +d[xLabel],
            share: +d[yLabel],
            change: +d[colorLabel] / 100,
        };
    });
    
}

const drawBars = async() => {

    //load in csv
    try {
        await fetchCSV();
    } catch (e) {
        console.log("there was an error loading the csv file:\n", e);
    }



    svg = d3.select("body").append("svg")
        .attr("width",  canvasWidth)
        .attr("height", canvasHeight);

    //------Title-----------
    svg.append("text")
        .attr("text-anchor", "middle")
        .attr("x", canvasWidth / 2)
        .attr("y", 30)
        .text("Avg. Share of Production for Renewables (EU Member States)");




    //------axis scales-------------
    let xScale = d3.scaleBand()
        .domain(data.map(d => d.year))
        .range([0, w]);
        
    
    let yScale = d3.scaleLinear()
        .domain([0, d3.max(data.map(d => d.share))])
        .range([h, 0]);
    
    let colorScale = d3.scaleLinear()
        .domain(d3.extent(data.map(d => d.change)))
        .range(["rgb(237,248,251)","rgb(0,109,44)"]);

    //prepare main container group
    let container_g = svg.append("g")
        .attr("transform", `translate(${marginX * 2 / 5}, ${marginY * 3 / 5})`)
        .attr("class", "bars-group");

    // Display the X-axis
    container_g.append("g")
        .attr("transform", `translate(0, ${h})`)
        .call(d3.axisBottom(xScale).tickSizeOuter(0))

    //label x
    svg.append("text")
        .attr("class", "x label")
        .attr("text-anchor", "middle")
        .attr("x", canvasWidth / 2)
        .attr("y", canvasHeight - 15)
        .text(xLabel)
    

    // Display the Y-axis
    container_g.append("g")
        .call(d3.axisLeft(yScale).tickFormat(d => d + "%"));

    //label y
    svg.append("text")
        .attr("class", "y label")
        .attr("transform", "rotate(-90)")
        .attr("text-anchor", "middle")
        .attr("x", canvasHeight / -2)
        .attr("y", 22)
        .text(yLabel)

    //show color legend
    var legendLinear = d3.legendColor()
        .shapeWidth(20)
        .orient('vertical')
        .scale(colorScale)
        .title(colorLabel)
        .titleWidth(marginX * 3 / 5)
        .labelFormat(d3.format(".1%"));;

    svg.append("g")
        .attr("class", "colorLegend")
        .attr("transform", `translate(${canvasWidth - marginX * 3 / 5 + 10},${marginY * 3 / 5})`)
        .call(legendLinear); 


    //prepare wrapper groups:
    let groups = container_g.selectAll(null)
        .data(data)
        .enter()
        .append("g")
        .attr("class", "bar-wrapper");

    //add bars to the groups
    groups.append("rect")
        .attr("class", "bar rect")
        .attr("fill", d => colorScale(d.change))
        .attr("x", d => xScale(d.year) + 2)
        .attr("y", d => yScale(d.share))
        .attr("width", w / data.length - 2)
        .attr("height", d => yScale(0) - yScale(d.share));

    //add text for hover interaction to the groups
    groups.append("text")
        .attr("class", "bar label")
        .attr("text-anchor", "middle")
        .attr("fill", d => d.change < 0.08 ? "black" : "white")
        .attr("x", d => xScale(d.year) + (w / data.length) / 2)
        .attr("y", d => yScale(d.share) + 20)
        .text(d => (d.change * 100).toFixed(1) + "%");
}

drawBars();
